/********************************************************************
   T/SBDIO-16 DCM Demo Utility for C3x. rev.1A
   (C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	This demo writes running '1' to DIO-A0..D3 outputs. C3x DSP on-chip
	SIO-0 port is used for this demo.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-3x, TORNADO-P3x and TORNADO-E3x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - no interrupt from DIO-16
		  - SIOX-Bus Device ID is '000'

***********************************************************************/

#define	_E3X_		1
#define	_T3X_		2
#define	_TP3X_		3


//~~~~~~~~~ TORNADO platform selector for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_E3X_

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E3X_SELECTED	(_TORNADO_==_E3X_)
#define	_T3X_SELECTED	(_TORNADO_==_T3X_)
#define	_TP3X_SELECTED	(_TORNADO_==_TP3X_)


#include	"math.h"
#include	"c30_def.h"
#include	"sdio_dsp.h"



//------ TORNADO platform specific functions --------------------
#if(_E3X_SELECTED)
#include	"te3x_dsp.h"
#define _REMOVE_SIOX_RESET	TE3X_set_pxsx_run(0x3)
#define _STB0_GC_DATA_		TE3X_PBUS_GC_DATA
#endif

#if(_T3X_SELECTED)
#include	"t3x_dsp.h"
#define _REMOVE_SIOX_RESET		;	// no T3x RESET removal for compatibility with T31
#define _STB0_GC_DATA_		T32L_STB0_GC_DATA
#define _STB1_GC_DATA_		T32L_STB1_GC_DATA
#define _IOSTB_GC_DATA_		T32L_IOSTB_GC_DATA
#endif

#if(_TP3X_SELECTED)
#include	"tp3x_dsp.h"
#define _REMOVE_SIOX_RESET	TP3X_set_pxsx_run(0x3)
#define _STB0_GC_DATA_		TP3X_PBUS_GC_DATA
#endif
//--------------------------------------------------------------

//------------------- SIO port defs ----------------------------
#define	_WAIT_SIO_TRDY		while(!((*SIO0_GC) & 2))
#define	_WAIT_SIO_RRDY		while(!((*SIO0_GC) & 1))

#define	_WRITE_SIO_DATA		(*SIO0_XD)
#define _READ_SIO_DATA		(*SIO0_RD)
//--------------------------------------------------------------




#define	SXBUS_DEV_ID		0

	//--- writes DATA to DIO-A0..D3
void set_dio_data(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
					   | TSBDIO16_DATA_WR_CMND
					   | (data & TSBDIO16_DATA_BITMASK)
			 		 )  ;
}

	//--- reads DIO DATA
unsigned get_dio_data()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data read command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
					   | TSBDIO16_DATA_RD_CMND
					 )  ;

	_WAIT_SIO_RRDY;

	return(_READ_SIO_DATA & TSBDIO16_DATA_BITMASK);
}




	//--- writes DIR
void set_dio_dir(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DIR write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_DIR_WR_CMND
			   | (data & TSBDIO16_DIR_BITMASK)
			 )  ;
}


	//--- reads DIR
unsigned get_dio_dir()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DIR read command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_DIR_RD_CMND
			 )  ;

	_WAIT_SIO_RRDY;

	return(_READ_SIO_DATA & TSBDIO16_DIR_BITMASK);
}


	//--- writes IRQ_SEL
void set_irq_sel(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_IRQ_SEL_WR_CMND
			   | (data & TSBDIO16_IRQ_SEL_BITMASK)
			 )  ;
}


	//--- reads IRQ_SEL
unsigned get_irq_sel()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send IRQ_SEL read command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_IRQ_SEL_RD_CMND
			 )  ;

	_WAIT_SIO_RRDY;

	return(_READ_SIO_DATA & TSBDIO16_IRQ_SEL_BITMASK);
}


	// CPU init
void init_cpu()
 {
   *STB0_C 	= _STB0_GC_DATA_;

	// configure STB1/IOSTB regs for T3x in order to support T32L boards
#if(_T3X_SELECTED)
   *STB1_C 	= _STB1_GC_DATA_;
   *IOSTB_C = _IOSTB_GC_DATA_;
#endif

	CACHE_ENABLE;
 }


void init_sio()
// configure C3x SIO-0 port and release SIOX-A RESET line
 {
	*SIO0_GC 	= TSBDIO16_C3X_SIO_GCRG_DATA  ;

	*SIO0_XC 	= TSBDIO16_C3X_SIO_XCRG_DATA  ;
	*SIO0_RC 	= TSBDIO16_C3X_SIO_RCRG_DATA  ;

	*SIO0_RXTC	= TSBDIO16_C3X_SIO_RXTCRG_DATA;
	*SIO0_RXTPER= TSBDIO16_C3X_SIO_RXTPERRG_DATA;
	*SIO0_RXTC	= TSBDIO16_C3X_SIO_RXTCRG_DATA | TSBDIO16_C3X_SIO_RXTCRG_GO_MASK;

	*SIO0_GC 	= TSBDIO16_C3X_SIO_GCRG_DATA | TSBDIO16_C3X_SIO_GCRG_GO_MASK ;

	_REMOVE_SIOX_RESET;
 }




void main()
{  int i,n;

	init_cpu();
	init_sio();

	// loop forever until error occurs
	while (1)
	 {
	 	//-- configure all pins as OUTPUTs
	  set_dio_dir(  TSBDIO16_DIRA_OUT
		      | TSBDIO16_DIRB_OUT
		      | TSBDIO16_DIRC_OUT
		      | TSBDIO16_DIRD_OUT
                     ) ;

		//-- run '1' via all DIO bits and verify via read-back
          for(i=0,n=1;i<16; i++,  n <<= 1)
	   { set_dio_data(n);
	     if(get_dio_data()!=n) goto error;
	   }

		//-- set all outputs to '1' in order to exclude '1->0'
		//   transitions during direction modification
	  set_dio_data(0xffff);	

		//-- run '1' via all DIR bits and verify via read-back
          for(i=0,n=1;i<4; i++,  n <<= 1)
	   { set_dio_dir(n);
	     if(get_dio_dir()!=n) goto error;
	   }
	   	//-- finally configure all DIO bits as INPUTS
	  set_dio_dir(0);

		//-- run '1' via all IRQ_SEL bits and verify via read-back
          for(i=0,n=1;i<2; i++,  n <<= 1)
	   { set_irq_sel(n);
	     if(get_irq_sel()!=n) goto error;
	   }
	   	//-- finally configure IRQ_SEL for DIO-A0
	  set_irq_sel(0);

	 }


 error:		// idle here on any error condition
 	while(1);

}

















