/********************************************************************
   T/SBDIO-16 DCM Demo Utility for C6x. rev.1A
   (C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	This demo writes running '1' to DIO-A0..D3 outputs.

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration jumpers:
          - no interrupt from DIO-16
	  - SIOX-Bus Device ID is '000'
	  - C6x McBSP-0 port is used (change this via _C6X_MCBSP_ def)

***********************************************************************/

#define	_E6X_		1
#define	_T6X_		2
#define	_TP6X_		3

#define	_C6X_MCBSP0_	0
#define	_C6X_MCBSP1_	1

//~~~~~~~~~ TORNADO and C6x McBSP port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_TP6X_

//!!!! modify this def in order to compile for McBSP-0 or McBSP-1 port
#define	_C6X_MCBSP_	_C6X_MCBSP0_
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E6X_SELECTED	(_TORNADO_==_E6X_)
#define	_T6X_SELECTED	(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED	(_TORNADO_==_TP6X_)


#define	_C6X_MCBSP0_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP0_)
#define	_C6X_MCBSP1_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP1_)


#include	"math.h"
#include	"c6x_def.h"
#include	"sdio_dsp.h"

//------- macros for McBSP port access ----------------------
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP1_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP1_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP1_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP1_DRR)
#endif


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


#define	SXBUS_DEV_ID		0

	//--- writes DATA to DIO-A0..D3
void set_dio_data(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_DATA_WR_CMND
			   | (data & TSBDIO16_DATA_BITMASK)
			 )  ;
}

	//--- reads DIO DATA
unsigned get_dio_data()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data read command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_DATA_RD_CMND
			 )  ;

	_WAIT_SIO_RRDY;

	return(_READ_SIO_DATA & TSBDIO16_DATA_BITMASK);
}




	//--- writes DIR
void set_dio_dir(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DIR write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_DIR_WR_CMND
			   | (data & TSBDIO16_DIR_BITMASK)
			 )  ;
}


	//--- reads DIR
unsigned get_dio_dir()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send DIR read command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_DIR_RD_CMND
			 )  ;

	_WAIT_SIO_RRDY;

	return(_READ_SIO_DATA & TSBDIO16_DIR_BITMASK);
}


	//--- writes IRQ_SEL
void set_irq_sel(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send data write command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_IRQ_SEL_WR_CMND
			   | (data & TSBDIO16_IRQ_SEL_BITMASK)
			 )  ;
}


	//--- reads IRQ_SEL
unsigned get_irq_sel()
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- send IRQ_SEL read command
	_WRITE_SIO_DATA =(   (SXBUS_DEV_ID << TSBDIO16_DEV_ID_SHIFT)
			   | TSBDIO16_IRQ_SEL_RD_CMND
			 )  ;

	_WAIT_SIO_RRDY;

	return(_READ_SIO_DATA & TSBDIO16_IRQ_SEL_BITMASK);
}


void init_cpu()
 {
	 // assume that C6x EMIF is initialized as default on reset
 }

void init_sio()
{
		//-- configure C6x McBSP port

#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSBDIO16_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSBDIO16_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP0_XCR  = TSBDIO16_C6X_MCBSP_XCR_DATA;
	*C6X_MCBSP0_RCR  = TSBDIO16_C6X_MCBSP_RCR_DATA;
	*C6X_MCBSP0_SRGR = TSBDIO16_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSBDIO16_C6X_MCBSP_SPCR_GO_MASK;
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
	*C6X_MCBSP1_SPCR = TSBDIO16_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR  = TSBDIO16_C6X_MCBSP_PCR_DATA;
	*C6X_MCBSP1_XCR  = TSBDIO16_C6X_MCBSP_XCR_DATA;
	*C6X_MCBSP1_RCR  = TSBDIO16_C6X_MCBSP_RCR_DATA;
	*C6X_MCBSP1_SRGR = TSBDIO16_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP1_SPCR |= TSBDIO16_C6X_MCBSP_SPCR_GO_MASK;
#endif

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}





void main()
{  int i,n;

	init_cpu();
	init_sio();

	while (1)
	 {
	 	//-- configure all pins as OUTPUTs
	  set_dio_dir(  TSBDIO16_DIRA_OUT
		      | TSBDIO16_DIRB_OUT
		      | TSBDIO16_DIRC_OUT
		      | TSBDIO16_DIRD_OUT
                     ) ;

		//-- run '1' via all DIO bits and verify via read-back
          for(i=0,n=1;i<16; i++,  n <<= 1)
	   { set_dio_data(n);
	     if(get_dio_data()!=n) goto error;
	   }

		//-- set all outputs to '1' in order to exclude '1->0'
		//   transitions during direction modification
	  set_dio_data(0xffff);	

		//-- run '1' via all DIR bits and verify via read-back
          for(i=0,n=1;i<4; i++,  n <<= 1)
	   { set_dio_dir(n);
	     if(get_dio_dir()!=n) goto error;
	   }
	   	//-- finally configure all DIO bits as INPUTS
	  set_dio_dir(0);

		//-- run '1' via all IRQ_SEL bits and verify via read-back
          for(i=0,n=1;i<2; i++,  n <<= 1)
	   { set_irq_sel(n);
	     if(get_irq_sel()!=n) goto error;
	   }
	   	//-- finally configure IRQ_SEL for DIO-A0
	  set_irq_sel(0);

	 }


 error:		// idle here on any error condition
 	while(1);

}

