/***************************************************************
  T/SBDIO-16 SIOX 16-bit Digital I/O DCM def for C3x/C6x DSP. Rev.1E
  (C) MicroLAB Systems Ltd, 1998-2000

  Notes:
    1. This file must be used as .H header file for 32-bit
       C3x/C6x DSP environments of host TORNADO DSP system or
       controller.
    2. All commands are adjusted for 24-bit commands and
       16-bit read-back data words.

****************************************************************/


//============= peripheral defs ===============

	//------- SIOX-Bus Devce ID
#define TSBDIO16_DEV_ID_BITMASK		0x00000e
#define TSBDIO16_DEV_ID_SHIFT		21

	//------- I/O data
#define TSBDIO16_DATA_BITMASK		0x00ffff

#define TSBDIO16_DATA_DIOA_BITMASK	0x00000f
#define TSBDIO16_DATA_DIOB_BITMASK	0x0000f0
#define TSBDIO16_DATA_DIOC_BITMASK	0x000f00
#define TSBDIO16_DATA_DIOD_BITMASK	0x00f000

#define TSBDIO16_DATA_DIOA0_BITMASK	0x000001
#define TSBDIO16_DATA_DIOA1_BITMASK	0x000002
#define TSBDIO16_DATA_DIOA2_BITMASK	0x000004
#define TSBDIO16_DATA_DIOA3_BITMASK	0x000008

#define TSBDIO16_DATA_DIOB0_BITMASK	0x000010
#define TSBDIO16_DATA_DIOB1_BITMASK	0x000020
#define TSBDIO16_DATA_DIOB2_BITMASK	0x000040
#define TSBDIO16_DATA_DIOB3_BITMASK	0x000080

#define TSBDIO16_DATA_DIOC0_BITMASK	0x000100
#define TSBDIO16_DATA_DIOC1_BITMASK	0x000200
#define TSBDIO16_DATA_DIOC2_BITMASK	0x000400
#define TSBDIO16_DATA_DIOC3_BITMASK	0x000800

#define TSBDIO16_DATA_DIOD0_BITMASK	0x001000
#define TSBDIO16_DATA_DIOD1_BITMASK	0x002000
#define TSBDIO16_DATA_DIOD2_BITMASK	0x004000
#define TSBDIO16_DATA_DIOD3_BITMASK	0x008000


	//------- direction control
#define TSBDIO16_DIR_BITMASK		0x00000f

#define TSBDIO16_DIRA_BITMASK		0x000001
#define TSBDIO16_DIRB_BITMASK		0x000002
#define TSBDIO16_DIRC_BITMASK		0x000004
#define TSBDIO16_DIRD_BITMASK		0x000008

#define TSBDIO16_DIRA_IN		0x000000
#define TSBDIO16_DIRA_OUT		0x000001
#define TSBDIO16_DIRB_IN		0x000000
#define TSBDIO16_DIRB_OUT		0x000002
#define TSBDIO16_DIRC_IN		0x000000
#define TSBDIO16_DIRC_OUT		0x000004
#define TSBDIO16_DIRD_IN		0x000000
#define TSBDIO16_DIRD_OUT		0x000008

	//------- host IRQ selector
#define TSBDIO16_IRQ_SEL_BITMASK	0x000003

#define TSBDIO16_IRQ_SEL_DIOA0		0x000000
#define TSBDIO16_IRQ_SEL_DIOA1		0x000001
#define TSBDIO16_IRQ_SEL_DIOA2		0x000002
#define TSBDIO16_IRQ_SEL_DIOA3		0x000003


//================ Host Commands ===============

	//---- read/write mis-commands
#define	TSBDIO16_NO_WR_CMND		0x000000
#define	TSBDIO16_NO_RD_CMND		0x000000

	//---- Data read/write commands
#define	TSBDIO16_DATA_WR_CMND		0x010000
#define	TSBDIO16_DATA_RD_CMND		0x040000

	//---- DIR rd/wr commands
#define	TSBDIO16_DIR_WR_CMND		0x020000
#define	TSBDIO16_DIR_RD_CMND		0x080000

	//---- IRQ-SEL rd/wr commands
#define	TSBDIO16_IRQ_SEL_WR_CMND	0x030000
#define	TSBDIO16_IRQ_SEL_RD_CMND	0x0c0000


//=============== SIO port configuration =====================
// recommended configuration for 60MHz C3x DSP:
//       XLen=24bits, Rlen=16bits
//       XCLK - Int,  RCLK - Ext
//       FSX - output (1), HS=0, XVAREN=0, XFSM=0
//       FSR - input, RVAREN=1, RFSM=0
//       CLKXP - active low (1), CLKRP - active high,
//       FSXP=FSRP - active low (1)

#define	TSBDIO16_C3X_SIO_GCRG_DATA			0x001B1244
#define	TSBDIO16_C3X_SIO_GCRG_GO_MASK		0x0c000000

#define	TSBDIO16_C3X_SIO_XCRG_DATA			0x111
#define	TSBDIO16_C3X_SIO_RCRG_DATA			0x111

#define TSBDIO16_C3X_SIO_RXTCRG_DATA		0xC
#define TSBDIO16_C3X_SIO_RXTCRG_GO_MASK		0x3
#define TSBDIO16_C3X_SIO_RXTPERRG_DATA		0x2


// recommended configuration for C6201@200MHz and C6701@167MHz DSP:
//       XLen=24bits  Rlen=16bits
//       XCLK - Int,  RCLK - Ext
//       FSX - output on DX load event, advance=+1 (one cycle prior data)
//       FSR - advance=0 (data framing)
//       CLKXP - active low (1), CLKRP - active high,
//       FSXP=FSRP - active low (1)
//	 CLKX = approx 15MHz

// SPCR: 0000 0000 0000 0000 0000 0000 0000 0000
// PCR:  0000 0000 0000 0000 0000 1010 0000 1110
// XCR:  0000 0000 0000 0001 0000 0000 1000 0000
// RCR:  0000 0000 0000 0000 0000 0000 0100 0000
// SRGR: 0010 0000 0000 0000 0000 0000 0000 1110

#define	TSBDIO16_C6X_MCBSP_SPCR_DATA	0x00000000
#define	TSBDIO16_C6X_MCBSP_PCR_DATA	0x00000a0e
#define	TSBDIO16_C6X_MCBSP_XCR_DATA	0x00010080
#define	TSBDIO16_C6X_MCBSP_RCR_DATA	0x00000040
#define	TSBDIO16_C6201_MCBSP_SRGR_DATA	0x2000000e
#define	TSBDIO16_C6701_MCBSP_SRGR_DATA	0x2000000b

#define	TSBDIO16_C6X_MCBSP_SPCR_GO_MASK	0x00410001

//===========================================================
